@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -81,21 +81,6 @@ bool pbm_getbit( FILE * f ) throw( Bitmap::Error )
   throw Bitmap::Error( "junk in pbm file where bits should be.\n" );
   }
 
-
-void mirror_left_right( std::vector< std::vector< bool > > data ) throw()
-  {
-  int height = data.size();
-  for( int row = 0; row < height; ++row )
-    reverse( data[row].begin(), data[row].end() );
-  }
-
-
-void mirror_top_bottom( std::vector< std::vector< bool > > data ) throw()
-  {
-  for( int u = 0, d = data.size() - 1; u < d; ++u, --d )
-    swap( data[u], data[d] );
-  }
-
 } // end namespace
 
 
@@ -250,35 +235,6 @@ bool Bitmap::find_rows_and_columns() throw()
   }
 
 
-void Bitmap::mirror_diagonal() throw()
-  {
-  int size = std::max( _width, _height );
-
-  if( _height < size )
-    {
-    data.resize( size );
-    for( int row = _height; row < size; ++row )
-      data[row].resize( size );
-    }
-  else if( _width < size )
-    for( int row = 0; row < _height; ++row )
-      data[row].resize( size );
-
-  for( int row = 0; row < size; ++row )
-    for( int col = 0; col < row; ++col )
-      {
-      bool b = data[row][col];
-      data[row][col] = data[col][row]; data[col][row] = b;
-      }
-
-  int tmp = _width; _width = _height; _height = tmp;
-  if( _height < size ) data.resize( _height );
-  else if( _width < size )
-    for( int row = 0; row < _height; ++row )
-      data[row].resize( _width );
-  }
-
-
 void Bitmap::floodfill4( const Bitmap & source, Bitmap & dest,
                          int row, int col ) throw()
   {
@@ -303,7 +259,7 @@ void Bitmap::floodfill4( const Bitmap & source, Bitmap & dest,
   }
 
 
-Bitmap::Bitmap( FILE * f, bool invert ) throw( Error )
+Bitmap::Bitmap( FILE * f, bool invert ) throw( Bitmap::Error )
   {
   char filetype = 0;
 
@@ -383,16 +339,7 @@ Bitmap::Bitmap( const Bitmap & source, int scale, int th ) throw()
       th >= scale * scale )
     Ocrad::internal_error( "bad parameter building a reduced Bitmap" );
 
-  if( th < 0 )
-    {
-    long dots = 0;
-    for( int row = 0; row < source._height; ++row )
-      for( int col = 0; col < source._width; ++col )
-        if( source.data[row][col] ) ++dots;
-    th = ( (scale * scale * dots) - 1 ) / (source._width * source._height);
-    std::fprintf( stderr, "automatic threshold = %d\n", th );
-    }
-
+  if( th < 0 ) th = ( ( scale * scale ) - 1 ) / 2;
   _height = source._height / scale;
   _width = source._width / scale;
 
@@ -414,7 +361,7 @@ Bitmap::Bitmap( const Bitmap & source, int scale, int th ) throw()
   }
 
 
-Bitmap::Bitmap( const Bitmap & source, Type t ) throw( Error )
+Bitmap::Bitmap( const Bitmap & source, Type t ) throw( Bitmap::Error )
   {
   _height = source._height; _width = source._width;
 
@@ -466,41 +413,7 @@ Bitmap::Bitmap( const std::vector< std::vector< int > > & matrix ) throw()
   }
 
 
-void Bitmap::draw_rectangle( const Rectangle & re ) throw()
-  {
-  int l = std::max( 0, re.left() );
-  int t = std::max( 0, re.top() );
-  int r = std::min( _width - 1, re.right() );
-  int b = std::min( _height - 1, re.bottom() );
-  if( l == re.left() ) for( int row = t; row <= b; ++row ) data[row][l] = true;
-  if( t == re.top() ) for( int col = l; col <= r; ++col ) data[t][col] = true;
-  if( r == re.right() ) for( int row = t; row <= b; ++row ) data[row][r] = true;
-  if( b == re.bottom() ) for( int col = l; col <= r; ++col ) data[b][col] = true;
-  }
-
-
-void Bitmap::draw_track( const Track & tr ) throw()
-  {
-  int l = std::max( 0, tr.left() );
-  int r = std::min( _width - 1, tr.right() );
-  if( l == r ) return;
-  if( l == tr.left() )
-    for( int row = tr.top(l); row <= tr.bottom(l); ++row )
-      if( row >= 0 && row < _height ) data[row][l] = true;
-  if( r == tr.right() )
-    for( int row = tr.top(r); row <= tr.bottom(r); ++row )
-      if( row >= 0 && row < _height ) data[row][r] = true;
-  for( int col = l; col <= r; ++col )
-    {
-    int row = tr.top( col );
-    if( row >= 0 && row < _height ) data[row][col] = true;
-    row = tr.bottom( col );
-    if( row >= 0 && row < _height ) data[row][col] = true;
-    }
-  }
-
-
-int Bitmap::analyse_layout( int layout_level ) throw()
+int Bitmap::analyse_layout( const int layout_level ) throw()
   {
   if( _rectangle_vector.size() != 0 )
     Ocrad::internal_error( "analyse_layout called twice" );
@@ -529,7 +442,7 @@ int Bitmap::analyse_layout( int layout_level ) throw()
   }
 
 
-bool Bitmap::only_this_rectangle( int i ) throw()
+bool Bitmap::only_this_rectangle( const int i ) throw()
   {
   if( i < 0 || i >= (int)_rectangle_vector.size() ) return false;
   if( _rectangle_vector.size() > 1 )
@@ -570,24 +483,3 @@ void Bitmap::save( FILE * f, char filetype ) const throw()
       }
     }
   }
-
-
-void Bitmap::transform( const Transformation & t ) throw()
-  {
-  switch( t.type() )
-    {
-    case Transformation::none:
-      break;
-    case Transformation::rotate90:
-      mirror_diagonal(); mirror_top_bottom( data ); break;
-    case Transformation::rotate180:
-      mirror_left_right( data ); mirror_top_bottom( data ); break;
-    case Transformation::rotate270:
-      mirror_diagonal(); mirror_left_right( data ); break;
-    case Transformation::mirror_lr: mirror_left_right( data ); break;
-    case Transformation::mirror_tb: mirror_top_bottom( data ); break;
-    case Transformation::mirror_d1: mirror_diagonal(); break;
-    case Transformation::mirror_d2:
-      mirror_diagonal(); mirror_left_right( data ); mirror_top_bottom( data ); break;
-    }
-  }
