@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,10 +37,13 @@
 
 namespace {
 
-// Date of this version: 2004-12-09
+// Date of this version: 2005-02-12
 
-const char * const program_version = "0.10";
-const char * const program_year    = "2004";
+const char * invocation_name = 0;
+const char * const Program_name    = "GNU Ocrad";
+const char * const program_name    = "ocrad";
+const char * const program_version = "0.11";
+const char * const program_year    = "2005";
 
 struct Input_control
   {
@@ -59,26 +62,31 @@ struct Input_control
 
 void show_version() throw()
   {
-  std::printf( "GNU Ocrad version %s\n", program_version );
+  std::printf( "%s version %s\n", Program_name, program_version );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
   std::printf( "This program is free software; you may redistribute it under the terms of\n" );
   std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
   }
 
 
-void show_error( const char * program_name, const char * msg ) throw()
+void show_error( const char * msg, int errcode = 0, bool help = false ) throw()
   {
-  if( msg && msg[0] != 0 ) std::fprintf( stderr, "ocrad: %s\n", msg );
-  if( program_name && program_name[0] != 0 )
-    std::fprintf( stderr, "Try `%s --help' for more information.\n", program_name );
+  if( msg && msg[0] != 0 )
+    {
+    std::fprintf( stderr, "%s: %s", program_name, msg );
+    if( errcode > 0 ) std::fprintf( stderr, ": %s", strerror( errcode ) );
+    std::fprintf( stderr, "\n" );
+    }
+  if( help && invocation_name && invocation_name[0] != 0 )
+    std::fprintf( stderr, "Try `%s --help' for more information.\n", invocation_name );
   }
 
 
-void show_help( const char * program_name, bool verbose ) throw()
+void show_help( bool verbose ) throw()
   {
-  std::printf( "GNU Ocrad, Optical Character Recognition program\n" );
-  std::printf( "Reads pbm file(s), or standard input, and sends text to standard output\n" );
-  std::printf( "\nUsage: %s [options] [files]\n", program_name );
+  std::printf( "%s - Optical Character Recognition program.\n", Program_name );
+  std::printf( "Reads pbm file(s), or standard input, and sends text to standard output.\n" );
+  std::printf( "\nUsage: %s [options] [files]\n", invocation_name );
   std::printf( "Options:\n");
   std::printf( "  -h, --help               display this help and exit\n");
   std::printf( "  -V, --version            output version information and exit\n");
@@ -90,6 +98,7 @@ void show_help( const char * program_name, bool verbose ) throw()
   std::printf( "  -i, --invert             invert image levels (white on black)\n");
   std::printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n");
   std::printf( "  -o <file>                place the output into <file>\n");
+  std::printf( "  -s, --scale=[-]<n>       scale input image by [1/]<n>\n");
   std::printf( "  -t, --transform=<name>   try `--transform=help' for a list of names\n");
   std::printf( "  -v, --verbose            be verbose\n");
   std::printf( "  -x <file>                export OCR Results File to <file>\n");
@@ -98,9 +107,8 @@ void show_help( const char * program_name, bool verbose ) throw()
     std::printf( "  -1, -4                   pbm output file type (debug)\n");
     std::printf( "  -C, --copy               'copy' input to output (debug)\n");
     std::printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n");
-    std::printf( "  -R <scale>               reduce input image by <scale> (debug)\n");
     std::printf( "  -S <type>                make a 'special file' (debug)\n");
-    std::printf( "  -T <threshold>           set threshold for -R option (debug)\n");
+    std::printf( "  -T <threshold>           set threshold for -s option (debug)\n");
     }
   std::printf( "\nReport bugs to bug-ocrad@gnu.org\n");
   }
@@ -123,6 +131,7 @@ int process_file( FILE *infile, const char * infile_name,
     Bitmap page_image( infile, input_control.invert );
 
     page_image.transform( input_control.transformation );
+    page_image.scale( input_control.scale, input_control.threshold );
     page_image.analyse_layout( input_control.layout_level );
     if( control.verbose ) std::fprintf( stderr, "number of text blocks %d\n",
                                 page_image.rectangles() );
@@ -135,16 +144,7 @@ int process_file( FILE *infile, const char * infile_name,
       return 1;
       }
 
-    if( input_control.scale != 0 )
-      {
-      if( control.outfile )
-        {
-        Bitmap reduced( page_image, input_control.scale, input_control.threshold );
-        reduced.save( control.outfile, control.filetype );
-        }
-      return 0;
-      }
-    else if( input_control.specialtype != 0 )
+    if( input_control.specialtype != 0 )
       {
       if( control.outfile )
         {
@@ -152,7 +152,7 @@ int process_file( FILE *infile, const char * infile_name,
         if( input_control.specialtype == 'v' ) t = Bitmap::vertical_histogram;
         else if( input_control.specialtype == 'h' ) t = Bitmap::horizontal_histogram;
         else if( input_control.specialtype == 'g' ) t = Bitmap::connected_ground;
-        else { show_error( 0, "bad special type" ); return 1; }
+        else { show_error( "bad special type" ); return 1; }
         Bitmap sb( page_image, t );
         sb.save( control.outfile, control.filetype );
         }
@@ -188,17 +188,27 @@ int process_file( FILE *infile, const char * infile_name,
 } // end namespace
 
 
+void Ocrad::internal_error( const char * msg ) throw()
+  {
+  char buf[80];
+  std::snprintf( buf, sizeof( buf ), "internal error: %s.\n", msg );
+  show_error( buf );
+  exit( 3 );
+  }
+
+
 // 'infile' contains the scanned image (in pbm format) to be converted
 // to text.
 // 'outfile' is the destination for the text version of the scanned
 // image. (or for a pbm file if debugging).
 // 'exportfile' is the Ocr Results File.
-int main( int argc, char *argv[] ) throw()
+int main( int argc, char * argv[] ) throw()
   {
   Input_control input_control;
   Control control;
-  char *outfile_name = 0, *exportfile_name = 0;
+  const char *outfile_name = 0, *exportfile_name = 0;
   bool append = false, force = false;
+  invocation_name = argv[0];
 
   // scan arguments
 
@@ -216,46 +226,47 @@ int main( int argc, char *argv[] ) throw()
       {"help", no_argument, 0, 'h'},
       {"invert", no_argument, 0, 'i'},
       {"layout", required_argument, 0, 'l'},
+      {"scale", required_argument, 0, 's'},
       {"transform", required_argument, 0, 't'},
       {"verbose", no_argument, 0, 'v'},
       {"version", no_argument, 0, 'V'},
       {0, 0, 0, 0}
       };
 
-    int c = getopt_long( argc, argv, "14CD:F:R:S:T:Vab:c:fhil:o:t:vx:",
+    int c = getopt_long( argc, argv, "14CD:F:S:T:Vab:c:fhil:o:s:t:vx:",
                          long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
     switch( c )
       {
-      case 0: break;
-      case '?': show_error( argv[0], 0 ); return 1;	// bad option
       case '1':
       case '4': control.filetype = c; break;
       case 'C': input_control.copy = true; break;
       case 'D': control.debug_level = std::strtol( optarg, 0, 0 ); break;
       case 'F': if( !control.set_format( optarg ) )
-                  { show_error( argv[0], "bad output format" ); return 1; }
+                  { show_error( "bad output format", 0, true ); return 1; }
                 break;
-      case 'R': input_control.scale = std::strtol( optarg, 0, 0 ); break;
       case 'S': input_control.specialtype = optarg[0]; break;
       case 'T': input_control.threshold = std::strtol( optarg, 0, 0 ); break;
       case 'V':	show_version(); return 0;
       case 'a': append = true; break;
       case 'b': input_control.rindex = std::strtol( optarg, 0, 0 ) - 1; break;
       case 'c': if( !control.charset.enable( optarg ) )
-                  { control.charset.show_error( optarg ); return 1; }
+                  { control.charset.show_error( program_name, optarg ); return 1; }
                 break;
       case 'f': force = true; break;
-      case 'h': show_help( argv[0], control.verbose ); return 0;
+      case 'h': show_help( control.verbose ); return 0;
       case 'i': input_control.invert = true; break;
       case 'l': input_control.layout_level = std::strtol( optarg, 0, 0 ); break;
       case 'o':	outfile_name = optarg; break;
+      case 's': input_control.scale = std::strtol( optarg, 0, 0 ); break;
       case 't': if( !input_control.transformation.set( optarg ) )
-                  { input_control.transformation.show_error( optarg ); return 1; }
+                  { input_control.transformation.show_error( program_name, optarg );
+                  return 1; }
                 break;
       case 'v': control.verbose = true; break;
       case 'x':	exportfile_name = optarg; break;
+      case '?': show_error( 0, 0, true ); return 1;	// bad option
       default: return 1;
       }
     } // end process options
@@ -289,13 +300,13 @@ int main( int argc, char *argv[] ) throw()
         }
       }
     std::fprintf( control.exportfile,
-                  "# Ocr Results File. Created by GNU Ocrad version %s\n",
-                  program_version );
+                  "# Ocr Results File. Created by %s version %s\n",
+                  Program_name, program_version );
     }
 
 // process any remaining command line arguments (input files)
   FILE *infile = (optind < argc) ? 0 : stdin;
-  char *infile_name = "-";
+  const char *infile_name = "-";
   int retval = 0;
   while( true )
     {
