@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -85,8 +85,8 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
   : Rectangle( r )
   {
   int cuts = block_matrix.size();
-  int mean_height[cuts];
-  std::vector< Block > pending[cuts], pending_tall[cuts], pending_short[cuts];
+  std::vector< int > mean_height( cuts, 0 );
+  std::vector< std::vector< Block > > pending( cuts ), pending_tall( cuts ), pending_short( cuts );
 
   // Classify blocks by height.
   for( int cut = 0; cut < cuts; ++cut )
@@ -110,7 +110,6 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
         if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
         ++height_distrib[h]; ++samples;
         }
-    mean_height[cut] = 0;
     int valid_samples = 0;
     for( unsigned int i = 0, count = 0; i < height_distrib.size(); ++i )
       {
@@ -282,19 +281,30 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
   for( int current_line = 0; current_line < lines(); ++current_line )
     {
     Textline & line1 = line( current_line );
-    int mw = line1.mean_width();
-    int space_width_limit = line1.space_width_limit( mw );
-    int mg = line1.mean_gap_width( space_width_limit );
+    const int mw = line1.mean_width();
+    const int space_width_limit = line1.space_width_limit( mw );
+    const int mg = line1.mean_gap_width( space_width_limit );
     if( mw < 2 || mg < 0 ) continue;
+    int spaces = 0, nospaces = 0, spsum = 0, nospsum = 0;
     for( int i = 1 ; i < line1.characters(); ++i )
       {
       Character & c1 = line1.character( i - 1 );
       Character & c2 = line1.character( i );
-      int gap = c2.left() - c1.right() - 1;
+      const int gap = c2.left() - c1.right() - 1;
       if( gap > space_width_limit )
         { if( line1.insert_space( i, true ) ) ++i; }
-      else if( gap >= mw || ( (3 * gap) > mw && gap > (2 * mg) ) )
-        if( line1.insert_space( i ) ) ++i;
+      else if( gap >= mw || ( 3 * gap > mw && gap > 2 * mg ) )
+        { ++spaces; spsum += gap; if( line1.insert_space( i ) ) ++i; }
+      else { ++nospaces; nospsum += gap; }
+      }
+    if( !spaces || !nospaces ) continue;
+    const int th = ( 2 * ( spsum / spaces ) ) + ( nospsum / nospaces ) + 3;
+    for( int i = 1 ; i < line1.characters(); ++i )
+      {
+      Character & c1 = line1.character( i - 1 );
+      Character & c2 = line1.character( i );
+      const int gap = c2.left() - c1.right() - 1;
+      if( 3 * gap > th ) { if( line1.insert_space( i ) ) ++i; }
       }
     }
 
