@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -56,8 +56,8 @@ void Textline::recognize2( const Charset & charset ) throw()
     if( !c.guesses() && c.blocks() == 2 &&
         c.block( 0 ).v_overlaps( c.block( 1 ) ) )
       {
-      Character c1( c.block( 0 ) ); c1.recognize1( charset, c.vcenter() );
-      Character c2( c.block( 1 ) ); c2.recognize1( charset, c.vcenter() );
+      Character c1( c.block( 0 ) ); c1.recognize1( charset, charbox( c1 ) );
+      Character c2( c.block( 1 ) ); c2.recognize1( charset, charbox( c2 ) );
       if( ( c1.guesses() && c2.guesses() ) ||
           Ocrad::similar( c1.height(), c2.height(), 20 ) )
         {
@@ -70,6 +70,19 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
     }
 
+  // remove speckles under the charbox' bottom of an unrecognized character
+  for( int i = 0; i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( !c.guesses() && c.blocks() == 2 &&
+        c.block( 0 ).size() > 10 * c.block( 1 ).size() &&
+        c.block( 1 ).top() > charbox( c ).bottom() )
+      {
+      Character c1( c.block( 0 ) ); c1.recognize1( charset, charbox( c1 ) );
+      if( c1.guesses() ) c = c1;
+      }
+    }
+
   // try to separate lightly merged characters
   // FIXME try all possible separation points
   for( int i = 0; i < characters(); ++i )
@@ -116,8 +129,8 @@ void Textline::recognize2( const Charset & charset ) throw()
         if( c1.includes_hcenter( bj ) ) c1.shift_block( bj );
         else if( c2.includes_hcenter( bj ) ) c2.shift_block( bj );
         }
-      c1.recognize1( charset, c.vcenter() );
-      c2.recognize1( charset, c.vcenter() );
+      c1.recognize1( charset, charbox( c1 ) );
+      c2.recognize1( charset, charbox( c2 ) );
       if( ( c1.guesses() && c2.guesses() ) ||
           ( ( c1.guesses() || c2.guesses() ) && c.width() > c.height() ) )
         {
@@ -137,17 +150,18 @@ void Textline::recognize2( const Charset & charset ) throw()
       Character c1( c );
       Block & b = c1.block( 0 );
       for( int j = b.blocks() - 1; j >= 0; --j )
-        if( 64 * b.block( j ).size() < b.size() ) b.delete_block( j );
+        if( 64 * b.block( j ).size() <= b.size() ||
+            16 * b.block( j ).height() <= b.height() ) b.delete_block( j );
       if( b.blocks() < c.block( 0 ).blocks() )
         {
-        c1.recognize1( charset, c1.vcenter() );
+        c1.recognize1( charset, charbox( c1 ) );
         if( c1.guesses() ) { c = c1; continue; }
         }
       if( b.blocks() == 1 && 16 * b.block( 0 ).size() < b.size() &&
           Ocrad::similar( b.height(), b.width(), 40 ) )
         {
         b.delete_block( 0 );
-        c1.recognize1( charset, c1.vcenter() );
+        c1.recognize1( charset, charbox( c1 ) );
         if( c1.guesses() ) { c = c1; continue; }
         }
       }
